@@ -331,10 +331,6 @@ gst_ffmpegviddec_close (GstFFMpegVidDec * ffmpegdec, gboolean reset)
     av_free (ffmpegdec->context->extradata);
     ffmpegdec->context->extradata = NULL;
   }
-  if (ffmpegdec->context->slice_offset) {
-    g_free (ffmpegdec->context->slice_offset);
-    ffmpegdec->context->slice_offset = NULL;
-  }
   if (reset) {
     if (avcodec_get_context_defaults3 (ffmpegdec->context,
             oclass->in_plugin) < 0) {
@@ -362,23 +358,10 @@ gst_ffmpegviddec_open (GstFFMpegVidDec * ffmpegdec)
     ffmpegdec->stride[i] = -1;
 
   ffmpegdec->opened = TRUE;
-  ffmpegdec->is_realvideo = FALSE;
 
   GST_LOG_OBJECT (ffmpegdec, "Opened libav codec %s, id %d",
       oclass->in_plugin->name, oclass->in_plugin->id);
 
-  switch (oclass->in_plugin->id) {
-    case AV_CODEC_ID_RV10:
-    case AV_CODEC_ID_RV30:
-    case AV_CODEC_ID_RV20:
-    case AV_CODEC_ID_RV40:
-      ffmpegdec->is_realvideo = TRUE;
-      break;
-    default:
-      GST_LOG_OBJECT (ffmpegdec, "Parser deactivated for format");
-      break;
-  }
-
   gst_ffmpegviddec_context_set_flags (ffmpegdec->context,
       CODEC_FLAG_OUTPUT_CORRUPT, ffmpegdec->output_corrupt);
 
@@ -815,7 +798,7 @@ gst_ffmpegviddec_get_buffer2 (AVCodecContext * context, AVFrame * picture,
   /* Fill avpicture */
   if (!gst_video_frame_map (&dframe->vframe, &ffmpegdec->pool_info,
           dframe->buffer, GST_MAP_READWRITE))
-    goto invalid_frame;
+    goto map_failed;
   dframe->mapped = TRUE;
 
   for (c = 0; c < AV_NUM_DATA_POINTERS; c++) {
@@ -850,30 +833,13 @@ gst_ffmpegviddec_get_buffer2 (AVCodecContext * context, AVFrame * picture,
 
   return 0;
 
-  /* fallbacks */
 no_dr:
-  {
-    GST_LOG_OBJECT (ffmpegdec, "direct rendering disabled, fallback alloc");
-    goto fallback;
-  }
-alloc_failed:
-  {
-    /* alloc default buffer when we can't get one from downstream */
-    GST_LOG_OBJECT (ffmpegdec, "alloc failed, fallback alloc");
-    goto fallback;
-  }
-invalid_frame:
-  {
-    /* alloc default buffer when we can't get one from downstream */
-    GST_LOG_OBJECT (ffmpegdec, "failed to map frame, fallback alloc");
-    gst_buffer_replace (&dframe->buffer, NULL);
-    goto fallback;
-  }
-fallback:
   {
     int c;
     int ret = avcodec_default_get_buffer2 (context, picture, flags);
 
+    GST_LOG_OBJECT (ffmpegdec, "direct rendering disabled, fallback alloc");
+
     for (c = 0; c < AV_NUM_DATA_POINTERS; c++) {
       ffmpegdec->stride[c] = picture->linesize[c];
     }
@@ -893,6 +859,21 @@ fallback:
 
     return ret;
   }
+alloc_failed:
+  {
+    GST_ELEMENT_ERROR (ffmpegdec, RESOURCE, FAILED,
+        ("Unable to allocate memory"),
+        ("The downstream pool failed to allocated buffer."));
+    return -1;
+  }
+map_failed:
+  {
+    GST_ELEMENT_ERROR (ffmpegdec, RESOURCE, OPEN_READ_WRITE,
+        ("Cannot access memory for read and write operation."),
+        ("The video memory allocated from downstream pool could not mapped for"
+            "read and write."));
+    return -1;
+  }
 duplicate_frame:
   {
     GST_WARNING_OBJECT (ffmpegdec, "already alloc'ed output buffer for frame");
@@ -1227,7 +1208,7 @@ get_output_buffer (GstFFMpegVidDec * ffmpegdec, GstVideoCodecFrame * frame)
   info = &ffmpegdec->output_state->info;
   if (!gst_video_frame_map (&vframe, info, frame->output_buffer,
           GST_MAP_READ | GST_MAP_WRITE))
-    goto alloc_failed;
+    goto map_failed;
 
   for (c = 0; c < AV_NUM_DATA_POINTERS; c++) {
     if (c < GST_VIDEO_INFO_N_PLANES (info)) {
@@ -1255,7 +1236,17 @@ get_output_buffer (GstFFMpegVidDec * ffmpegdec, GstVideoCodecFrame * frame)
   /* special cases */
 alloc_failed:
   {
-    GST_DEBUG_OBJECT (ffmpegdec, "allocation failed");
+    GST_ELEMENT_ERROR (ffmpegdec, RESOURCE, FAILED,
+        ("Unable to allocate memory"),
+        ("The downstream pool failed to allocated buffer."));
+    return ret;
+  }
+map_failed:
+  {
+    GST_ELEMENT_ERROR (ffmpegdec, RESOURCE, OPEN_READ_WRITE,
+        ("Cannot access memory for read and write operation."),
+        ("The video memory allocated from downstream pool could not mapped for"
+            "read and write."));
     return ret;
   }
 not_negotiated:
@@ -1306,24 +1297,6 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
    * else we might skip a reference frame */
   gst_ffmpegviddec_do_qos (ffmpegdec, frame, &mode_switch);
 
-  if (ffmpegdec->is_realvideo && data != NULL) {
-    gint slice_count;
-    gint i;
-
-    /* setup the slice table for realvideo */
-    if (ffmpegdec->context->slice_offset == NULL)
-      ffmpegdec->context->slice_offset = g_malloc (sizeof (guint32) * 1000);
-
-    slice_count = (*data++) + 1;
-    ffmpegdec->context->slice_count = slice_count;
-
-    for (i = 0; i < slice_count; i++) {
-      data += 4;
-      ffmpegdec->context->slice_offset[i] = GST_READ_UINT32_LE (data);
-      data += 4;
-    }
-  }
-
   if (frame) {
     /* save reference to the timing info */
     ffmpegdec->context->reordered_opaque = (gint64) frame->system_frame_number;
@@ -1408,6 +1381,16 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
     *ret = get_output_buffer (ffmpegdec, out_frame);
     gst_buffer_unref (tmp);
   }
+#ifndef G_DISABLE_ASSERT
+  else {
+    GstVideoMeta *vmeta = gst_buffer_get_video_meta (out_frame->output_buffer);
+    if (vmeta) {
+      GstVideoInfo *info = &ffmpegdec->output_state->info;
+      g_assert (vmeta->width == GST_VIDEO_INFO_WIDTH (info));
+      g_assert (vmeta->height == GST_VIDEO_INFO_HEIGHT (info));
+    }
+  }
+#endif
   gst_object_unref (pool);
 
   if (G_UNLIKELY (*ret != GST_FLOW_OK))
@@ -1835,10 +1818,13 @@ gst_ffmpegviddec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
   /* If we have videometa, we never have to copy */
   if (have_videometa && have_pool && have_alignment &&
       gst_ffmpegviddec_can_direct_render (ffmpegdec)) {
-    gst_ffmpegvideodec_prepare_dr_pool (ffmpegdec, pool, &state->info, config);
+    GstStructure *config_copy = gst_structure_copy (config);
+
+    gst_ffmpegvideodec_prepare_dr_pool (ffmpegdec, pool, &state->info,
+        config_copy);
 
     /* FIXME validate and retry */
-    if (gst_buffer_pool_set_config (pool, config)) {
+    if (gst_buffer_pool_set_config (pool, config_copy)) {
       GstFlowReturn ret;
       GstBuffer *tmp;
 
@@ -1863,16 +1849,20 @@ gst_ffmpegviddec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
             gst_object_unref (ffmpegdec->internal_pool);
           ffmpegdec->internal_pool = gst_object_ref (pool);
           ffmpegdec->pool_info = state->info;
+          gst_structure_free (config);
           goto done;
         }
       }
     }
   }
 
-  if (have_videometa && ffmpegdec->internal_pool) {
+  if (have_videometa && ffmpegdec->internal_pool
+      && ffmpegdec->pool_width == state->info.width
+      && ffmpegdec->pool_height == state->info.height) {
     update_pool = TRUE;
     gst_object_unref (pool);
     pool = gst_object_ref (ffmpegdec->internal_pool);
+    gst_structure_free (config);
     goto done;
   }
 
@@ -1884,6 +1874,8 @@ gst_ffmpegviddec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
     if (gst_buffer_pool_config_validate_params (config, state->caps, size, min,
             max)) {
       working_pool = gst_buffer_pool_set_config (pool, config);
+    } else {
+      gst_structure_free (config);
     }
 
     if (!working_pool) {
