@@ -40,35 +40,38 @@ struct _GstFFMpegVidDec
   gint stride[AV_NUM_DATA_POINTERS];
   gboolean opened;
 
+  /* current output pictures */
+  enum PixelFormat pic_pix_fmt;
+  gint pic_width;
+  gint pic_height;
+  gint pic_par_n;
+  gint pic_par_d;
+  gint pic_interlaced;
   /* current context */
-  enum PixelFormat ctx_pix_fmt;
-  gint ctx_width;
-  gint ctx_height;
-  gint ctx_par_n;
-  gint ctx_par_d;
   gint ctx_ticks;
   gint ctx_time_d;
   gint ctx_time_n;
-  gint ctx_interlaced;
   GstBuffer *palette;
 
   guint8 *padded;
   guint padded_size;
 
-  gboolean current_dr;          /* if direct rendering is enabled */
-
   /* some properties */
   enum AVDiscard skip_frame;
   gint lowres;
   gboolean direct_rendering;
   gboolean debug_mv;
   int max_threads;
-
-  gboolean is_realvideo;
+  gboolean output_corrupt;
 
   GstCaps *last_caps;
 
-  GSList *opaques;
+  /* Internally used for direct rendering */
+  GstBufferPool *internal_pool;
+  gint pool_width;
+  gint pool_height;
+  enum PixelFormat pool_format;
+  GstVideoInfo pool_info;
 };
 
 typedef struct _GstFFMpegVidDecClass GstFFMpegVidDecClass;
